import typing
from odoorpc import fields as fields
from odoorpc.models import Model as Model
from odoorpc.tools import v as v

if typing.TYPE_CHECKING:
    from odoorpc import ODOO

FIELDS_RESERVED: list[str]

class Environment:
    def __init__(self, odoo: ODOO, db: str, uid: int, context: str) -> None: ...
    @property
    def dirty(self): ...
    @property
    def context(self) -> dict: ...
    @property
    def db(self) -> str: ...
    def commit(self) -> None: ...
    def invalidate(self) -> None: ...
    @property
    def lang(self) -> str: ...
    def ref(self, xml_id): ...
    @property
    def uid(self) -> int: ...
    @property
    def user(self): ...
    @property
    def registry(self): ...
    def __getitem__(self, model): ...
    def __call__(self, context: dict | None = None): ...
    def __contains__(self, model) -> bool: ...



from .error import InternalError as InternalError
from _typeshed import Incomplete
from collections import MutableMapping
from collections.abc import Generator

MATCH_VERSION: Incomplete

class Config(MutableMapping):
    def __init__(self, odoo, options) -> None: ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value) -> None: ...
    def __delitem__(self, key) -> None: ...
    def __iter__(self): ...
    def __len__(self) -> int: ...

def clean_version(version): ...
def v(version): ...
def get_encodings(hint_encoding: str = "utf-8") -> Generator[Incomplete, None, None]: ...



import typing
from _typeshed import Incomplete
from odoorpc.models import IncrementalRecords as IncrementalRecords, Model as Model

if typing.TYPE_CHECKING:
    from collections.abc import Iterable

def is_int(value) -> bool: ...
def is_string(value) -> bool: ...
def odoo_tuple_in(iterable: Iterable) -> bool: ...
def tuples2ids(tuples: Iterable[tuple], ids: list) -> list: ...
def records2ids(iterable: Iterable[Model]) -> list[int]: ...

class BaseField:
    name: str
    type: bool
    string: bool
    size: bool
    required: bool
    readonly: bool
    help: bool
    states: bool
    def __init__(self, name, data) -> None: ...
    def __get__(self, instance, owner) -> None: ...
    def __set__(self, instance, value) -> None: ...
    def check_required(self, value) -> bool: ...
    def check_value(self, value): ...
    def store(self, record, value) -> None: ...

class Binary(BaseField):
    def __init__(self, name: str, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...

class Boolean(BaseField):
    def __init__(self, name: str, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...

class Char(BaseField):
    def __init__(self, name: str, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...

class Date(BaseField):
    pattern: str
    def __init__(self, name: str, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_value(self, value): ...

class Datetime(BaseField):
    pattern: str
    def __init__(self, name: str, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_value(self, value): ...

class Float(BaseField):
    def __init__(self, name: str, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_required(self, value): ...

class Integer(BaseField):
    def __init__(self, name: str, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_required(self, value): ...

class Selection(BaseField):
    selection: Incomplete
    def __init__(self, name: str, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_value(self, value): ...

class Many2many(BaseField):
    relation: Incomplete
    context: Incomplete
    domain: Incomplete
    def __init__(self, name, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_value(self, value): ...
    def store(self, record, value) -> None: ...

class Many2one(BaseField):
    relation: Incomplete
    context: Incomplete
    domain: Incomplete
    def __init__(self, name, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_value(self, value): ...

class One2many(BaseField):
    relation: Incomplete
    context: Incomplete
    domain: Incomplete
    def __init__(self, name, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_value(self, value): ...
    def store(self, record, value) -> None: ...

class Reference(BaseField):
    context: Incomplete
    domain: Incomplete
    selection: Incomplete
    def __init__(self, name, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...
    def check_value(self, value): ...

class Text(BaseField):
    def __init__(self, name, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...

class Html(Text):
    def __init__(self, name, data) -> None: ...

class Unknown(BaseField):
    def __init__(self, name, data) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...

TYPES_TO_FIELDS: Incomplete

def generate_field(name, data): ...



from _typeshed import Incomplete

def encode_data(data): ...
def decode_data(data): ...

LOG_HIDDEN_JSON_PARAMS: Incomplete
LOG_JSON_SEND_MSG: str
LOG_JSON_RECV_MSG: str
LOG_HTTP_SEND_MSG: str
LOG_HTTP_RECV_MSG: str
logger: Incomplete

def get_json_log_data(data): ...

class Proxy:
    def __init__(self, host, port, timeout: int = 120, ssl: bool = False, opener: Incomplete | None = None) -> None: ...
    def __getattr__(self, name): ...
    def __getitem__(self, url): ...

class ProxyJSON(Proxy):
    def __init__(
        self,
        host,
        port,
        timeout: int = 120,
        ssl: bool = False,
        opener: Incomplete | None = None,
        deserialize: bool = True,
    ) -> None: ...
    def __call__(self, url, params: Incomplete | None = None): ...

class ProxyHTTP(Proxy):
    def __call__(self, url, data: Incomplete | None = None, headers: Incomplete | None = None): ...

class URLBuilder:
    def __init__(self, rpc, url: Incomplete | None = None) -> None: ...
    def __getattr__(self, path): ...
    def __getitem__(self, path): ...
    def __call__(self, **kwargs): ...



from odoorpc import error as error
from odoorpc.tools import v as v

def encode2bytes(data): ...

class DB:
    def __init__(self, odoo) -> None: ...
    def dump(self, password, db, format_: str = "zip"): ...
    def change_password(self, password, new_password) -> None: ...
    def create(self, password, db, demo: bool = False, lang: str = "en_US", admin_password: str = "admin") -> None: ...
    def drop(self, password, db): ...
    def duplicate(self, password, db, new_db) -> None: ...
    def list(self): ...
    def restore(self, password, db, dump, copy: bool = False) -> None: ...



def get_all(rc_file: str = "~/.odoorpcrc"): ...
def get(name, rc_file: str = "~/.odoorpcrc"): ...
def save(name, data, rc_file: str = "~/.odoorpcrc") -> None: ...
def remove(name, rc_file: str = "~/.odoorpcrc") -> None: ...



from _typeshed import Incomplete
from odoorpc.rpc import error as error, jsonrpclib as jsonrpclib

class Connector:
    host: Incomplete
    port: Incomplete
    version: Incomplete
    def __init__(self, host, port: int = 8069, timeout: int = 120, version: Incomplete | None = None) -> None: ...
    @property
    def ssl(self): ...
    @property
    def timeout(self): ...
    @timeout.setter
    def timeout(self, timeout) -> None: ...

class ConnectorJSONRPC(Connector):
    deserialize: Incomplete
    def __init__(
        self,
        host,
        port: int = 8069,
        timeout: int = 120,
        version: Incomplete | None = None,
        deserialize: bool = True,
        opener: Incomplete | None = None,
    ) -> None: ...
    @property
    def proxy_json(self): ...
    @property
    def proxy_http(self): ...
    @property
    def timeout(self): ...
    @timeout.setter
    def timeout(self, timeout) -> None: ...

class ConnectorJSONRPCSSL(ConnectorJSONRPC):
    def __init__(
        self,
        host,
        port: int = 8069,
        timeout: int = 120,
        version: Incomplete | None = None,
        deserialize: bool = True,
        opener: Incomplete | None = None,
    ) -> None: ...
    @property
    def ssl(self): ...

PROTOCOLS: Incomplete



from _typeshed import Incomplete

__all__ = ["Model"]

class IncrementalRecords:
    tuples: Incomplete
    def __init__(self, tuples) -> None: ...

class MetaModel(type):
    def __getattr__(cls, method): ...
    @property
    def env(cls): ...

BaseModel: MetaModel

class Model(BaseModel):
    __metaclass__ = MetaModel
    def __init__(self) -> None: ...
    @property
    def env(self): ...
    @property
    def id(self): ...
    @property
    def ids(self): ...
    @classmethod
    def browse(cls, ids): ...
    @classmethod
    def with_context(cls, *args, **kwargs): ...
    @classmethod
    def with_env(cls, env): ...
    def __getattr__(self, method): ...
    def __getitem__(self, key): ...
    def __int__(self) -> int: ...
    def __eq__(self, other): ...
    __hash__: Incomplete
    def __ne__(self, other): ...
    def __iter__(self): ...
    def __nonzero__(self): ...
    def __len__(self) -> int: ...
    def __iadd__(self, records): ...
    def __isub__(self, records): ...



from _typeshed import Incomplete

class ConnectorError(BaseException):
    message: Incomplete
    odoo_traceback: Incomplete
    def __init__(self, message, odoo_traceback: Incomplete | None = None) -> None: ...



from _typeshed import Incomplete
from typing import Literal, TYPE_CHECKING
from odoorpc import error as error, rpc as rpc, session as session, tools as tools
from odoorpc.db import DB as DB
from odoorpc.env import Environment as Environment
from odoorpc.report import Report as Report

if TYPE_CHECKING:
    from odoorpc.models import Model

class ODOO:
    def __init__(
        self,
        host: str = "localhost",
        protocol: str = "jsonrpc",
        port: int = 8069,
        timeout: int = 120,
        version: Incomplete | None = None,
        opener: Incomplete | None = None,
    ) -> None: ...
    @property
    def config(self) -> dict: ...
    @property
    def version(self) -> str: ...
    @property
    def db(self) -> DB: ...
    @property
    def report(self) -> Report: ...
    host: str
    port: int
    protocol: Literal["jsonrpc", "jsonrpc+ssl"]
    @property
    def env(self) -> Environment: ...
    def json(self, url, params): ...
    def http(self, url, data: Incomplete | None = None, headers: Incomplete | None = None): ...
    def login(self, db, login: str = "admin", password: str = "admin") -> None: ...
    def logout(self): ...
    def close(self): ...
    def execute(self, model: str, method, *args): ...
    def execute_kw(self, model: str, method, args: Incomplete | None = None, kwargs: Incomplete | None = None): ...
    def exec_workflow(self, model: str, record_id, signal): ...
    def save(self, name, rc_file: str = "~/.odoorpcrc") -> None: ...
    @classmethod
    def load(cls, name, rc_file: str = "~/.odoorpcrc"): ...
    @classmethod
    def list(cls, rc_file: str = "~/.odoorpcrc"): ...
    @classmethod
    def remove(cls, name, rc_file: str = "~/.odoorpcrc"): ...



from _typeshed import Incomplete
from odoorpc.tools import get_encodings as get_encodings, v as v

def encode2bytes(data): ...

class Report:
    def __init__(self, odoo) -> None: ...
    def download(self, name: str, ids: list[int], datas: Incomplete | None = None, context: Incomplete | None = None): ...
    def list(self) -> dict: ...
